@@ -57,20 +57,28 @@ class RelationTestCase(TestCase):
 
         def f():
             return cellvalue
-        self.chkrel(f.__closure__[0], cellvalue, '%s->ob_ref')
+        self.chkrel(f.__closure__[0], cellvalue, '%s.cell_contents')
 
     def test_code_relation(self):
         def f():
-            a = 3
+            a = 1000
             return self, a
         co = f.__code__
         # xxx brittle test but catches a bug
-        self.chkpath(co, 3, '%s.co_consts[1]')
+        # Python 3.14 added LOAD_SMALL_INT opcode. This value must be
+        # sufficiently large.
+        if self.version_info < (3, 14):
+            self.chkpath(co, 1000, '%s.co_consts[1]')
+        else:
+            self.chkpath(co, 1000, '%s.co_consts[0]')
         # commented in notes Sep 27 2004
-        self.chkrelattr(co, 'co_code', 'co_consts', 'co_names', 'co_varnames',
-                        'co_freevars', 'co_cellvars', 'co_filename', 'co_name',
-                        'co_lnotab')
-
+        relAttr = ('co_code', 'co_consts', 'co_names',
+                   'co_filename', 'co_name', 'co_linetable')
+        if self.version_info < (3, 11):
+            relAttr += ('co_varnames', 'co_freevars', 'co_cellvars')
+        else:
+            relAttr += ('co_exceptiontable', 'co_qualname')
+        self.chkrelattr(co, *relAttr)
 
     def test_dict_relation(self):
         k1 = 'k1'
@@ -102,7 +110,9 @@ class RelationTestCase(TestCase):
         f = inspect.currentframe()
         f.f_trace = lambda: None
         self.chkrelattr(f, 'f_back', 'f_builtins', 'f_code', 'f_globals',
-                        'f_locals', 'f_trace')
+                        'f_trace')
+        if self.version_info < (3, 13):
+            self.chkrelattr(f, 'f_locals')
 
         a = []
         # The representation of local variables is how they may be accessed
@@ -227,7 +237,10 @@ class RelationTestCase(TestCase):
         self.chkpath(w, w.b, '%s.b')
         self.chkpath(w, w.c, '%s.c')
         self.chkpath(w, w.d, '%s.d')
-        self.chkpath(w, w.e, "%s.__dict__['e']")
+        if self.version_info < (3, 14):
+            self.chkpath(w, w.e, "%s.__dict__['e']")
+        else:
+            self.chkpath(w, w.e, "%s.e")
 
         class R(object):
             rvar = []
@@ -242,7 +255,10 @@ class RelationTestCase(TestCase):
         self.chkrelattr(s, '__dict__', 'a', 'b', 'c')
         self.chkpath(s, s.a, '%s.a')
         self.chkpath(s, s.b, '%s.b')
-        self.chkpath(s, s.c, "%s.__dict__['c']")
+        if self.version_info < (3, 14):
+            self.chkpath(s, s.c, "%s.__dict__['c']")
+        else:
+            self.chkpath(s, s.c, "%s.c")
 
         # Class variables are not directly related- should they be that?
         # Possibly, but the compression could as well be done in Python.
@@ -293,38 +309,48 @@ class RelationTestCase(TestCase):
 
 class RootTestCase(TestCase):
     def test_1(self):
+        import gc
         import sys
         import builtins
         root = self.View.root
         # Interpreter attributes
 
         rel = str(self.relation(root, sys.modules))
-
-        self.assertTrue(eval(rel % 'root') is sys.modules)
         self.aseq(rel, '%s.i0_modules')
+        self.assertTrue(eval(rel % 'root') is sys.modules)
 
         rel = str(self.relation(root, sys.__dict__))
-        self.assertTrue(eval(rel % 'root') is sys.__dict__)
         self.aseq(rel, '%s.i0_sysdict')
+        self.assertTrue(eval(rel % 'root') is sys.__dict__)
 
         rel = str(self.relation(root, builtins.__dict__))
-        self.assertTrue(eval(rel % 'root') is builtins.__dict__)
         self.aseq(rel, '%s.i0_builtins')
+        self.assertTrue(eval(rel % 'root') is builtins.__dict__)
 
         for name in "codec_search_path", "codec_search_cache", "codec_error_registry":
             attr = "i0_%s" % name
             rel = str(self.relation(root, getattr(root, attr)))
             self.aseq(rel, '%%s.%s' % attr)
 
+        # This is fragilely referred to by gc's dict via PyModule_AddObjectRef.
+        # but also stored by the interpreter state via _is.gc.callbacks
+        # If this test becomes no relation we need to add this to rootstate.
+        rel = self.chkpath(root, gc.callbacks,
+                           "%s.i0_modules['gc'].__dict__['callbacks']")
+
         # Thread attributes
 
         try:
             1/0
         except ZeroDivisionError:
             exc_type, exc_value, exc_traceback = sys.exc_info()
-            for name in 'exc_type', 'exc_value', 'exc_traceback':
-                rel = str(self.relation(root, eval(name)))
-                self.asis(eval(rel % 'root'), eval(name))
+            if sys.version_info >= (3, 11):
+                rel = str(self.relation(root, exc_value))
+                self.asis(eval(rel % 'root'), exc_value)
+            else:
+                for name in 'exc_type', 'exc_value', 'exc_traceback':
+                    rel = str(self.relation(root, eval(name)))
+                    self.asis(eval(rel % 'root'), eval(name))
 
             # There are more, untested, attributes, but the code is farily regular...
             # More complication is to do with frames which I concentrate on for now.
@@ -357,7 +383,7 @@ class RootTestCase(TestCase):
                 1/0
             except ZeroDivisionError:
                 exc_type, exc_value, exc_traceback = sys.exc_info()
-                self.exc_traceback = exc_traceback
+                self.exc_value = exc_value
                 self.sync = 1
                 while self.sync:
                     pass
@@ -367,9 +393,9 @@ class RootTestCase(TestCase):
         _thread.start_new_thread(task, (self,))
         while not self.sync:
             pass
-        exc_traceback = self.exc_traceback
-        rel = str(self.relation(root, exc_traceback))
-        self.asis(eval(rel % 'root'), exc_traceback)
+        exc_value = self.exc_value
+        rel = str(self.relation(root, exc_value))
+        self.asis(eval(rel % 'root'), exc_value)
         self.sync = 0
         while not self.sync:
             pass
@@ -382,48 +408,6 @@ class RootTestCase(TestCase):
         while not self.sync:
             pass
 
-    def test_secondary_interpreter(self):
-        try:
-            import _thread
-        except ImportError:
-            print('threading not enabled - skipping test')
-            return
-
-        import_remote = """\
-import sys
-import _thread
-import time
-
-def task():
-    time.sleep(1)
-
-    self.sysdict = sys.__dict__
-    self.sync = 1
-
-    while self.sync:
-        pass
-
-_thread.start_new_thread(task, ())
-"""
-
-        self.sync = 0
-        thid = self.heapy.heapyc.interpreter(import_remote, {'self': self})
-
-        root = self.View.root
-
-        import sys
-        sysdict = sys.__dict__
-        rel = str(self.relation(root, sysdict))
-        self.aseq(rel, '%s.i0_sysdict')
-
-        while not self.sync:
-            pass
-
-        rel = str(self.relation(root, self.sysdict))
-        self.aseq(rel, '%s.i1_sysdict')
-
-        self.sync = 0
-
 
 class PathTestCase(TestCase):
     def makegraph(self, width, length):
@@ -564,16 +548,14 @@ class PathTestCase(TestCase):
         src, dst = self.makegraph(width, length)
         p = self.shpaths(dst, src)
         p.maxpaths = 2
-        p.pp()
-        p.more()
-        self.aseq(output.getvalue(), """\
+        self.aseq(str(p), """\
  0: Src[0][0][0][0]
  1: Src[0][0][0][1]
-<... 14639 more paths ...>
+<... 14639 more paths ...>""")
+        self.aseq(str(p.more), """\
  2: Src[0][0][0][2]
  3: Src[0][0][0][3]
-<... 14637 more paths ...>
-""")
+<... 14637 more paths ...>""")
 
     def test_subscript(self):
         # Test subscripting
@@ -598,30 +580,26 @@ class PathTestCase(TestCase):
 class MultiTestCase(TestCase):
     def test_pp(self):
         # Test printing of multi relations
-        self.Path.output = self.Path._root.io.StringIO()
         iso = self.iso
         dst = [[], []]
         src = iso(dst[:]*2)
         dst = [iso(x) for x in dst]
         p = self.Path.shpgraph(dst, src)
-        p.pp()
-        p = self.Path.shpgraph(dst, src, srcname='A', dstname='B')
-        p.pp()
-
-        self.aseq(self.Path.output.getvalue(), """\
+        self.aseq(str(p), """\
 --- Dst[0] ---
  0: Src[0]
  1: Src[2]
 --- Dst[1] ---
  0: Src[1]
- 1: Src[3]
+ 1: Src[3]""")
+        p = self.Path.shpgraph(dst, src, srcname='A', dstname='B')
+        self.aseq(str(p), """\
 --- B[0] ---
  0: A[0]
  1: A[2]
 --- B[1] ---
  0: A[1]
- 1: A[3]
-""")
+ 1: A[3]""")
 
 
 class AvoidTestCase(TestCase):
@@ -763,7 +741,11 @@ class NewTestCase(TestCase):
         b = list(it)
         self.aseq(str(a), str(b))
 
-        self.aseq(o.getvalue(), """\
+        # The sort order is based on the source set's byid partition (see
+        # Path.PathsIter.reset), which sorts based on the size, then render
+        # (see Part.IdentityPartition.__init__)
+        if sys.getsizeof(y) > sys.getsizeof(ob):
+            self.aseq(o.getvalue(), """\
  0: hpy().Root.i0_sysdict
  0: Src.i0_modules['sys'].__dict__
  0: hpy().Root.i0_sysdict
@@ -779,6 +761,23 @@ class NewTestCase(TestCase):
  0: <1 list: <address>*1>[0]
  1: <1 __main__.O: <address>>.x
 """.replace('__main__', self.__module__))
+        else:
+            self.aseq(o.getvalue(), """\
+ 0: hpy().Root.i0_sysdict
+ 0: Src.i0_modules['sys'].__dict__
+ 0: hpy().Root.i0_sysdict
+ 0: hpy().Root.i0_sysdict
+ 0: Src[0][0][0]
+ 0: Src[0][0][0]
+ 0: Src[0]
+ 0: Src.x
+ 0: <1 list: <address>*1>[0]
+ 0: <1 list: <address>*1>[0]
+ 0: <1 __main__.O: <address>>.x
+ 1: <1 list: <address>*1>[0]
+ 0: <1 __main__.O: <address>>.x
+ 1: <1 list: <address>*1>[0]
+""".replace('__main__', self.__module__))
 
     def test_2(self):
         # To assist interactivity,
@@ -932,6 +931,19 @@ class NewTestCase(TestCase):
 10: Src[10]
 """)
 
+    def test_comparison(self):
+        # Test that non-compariable keys won't crash
+        # output order may be arbitrary not the output is not tested
+
+        iso = self.iso
+        dst = []
+
+        shp = iso(dst).get_shpaths(iso({0: dst, '': dst}))
+        str(shp)
+
+        shp = iso(dst).get_shpaths(iso({object(): dst, object(): dst}))
+        str(shp)
+
 
 def run_test(case, debug=0):
     support.run_unittest(case, debug)
