@@ -158,6 +158,10 @@ static struct PyMemberDef is_members[] = {
 #endif
 
     MEMBER(dict),
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    MEMBER(sysdict_copy),
+#endif
     MEMBER(builtins_copy),
 
 #ifdef HAVE_FORK
@@ -165,14 +169,13 @@ static struct PyMemberDef is_members[] = {
     MEMBER(after_forkers_parent),
     MEMBER(after_forkers_child),
 #endif
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 10
-    MEMBER(pyexitmodule),
-#endif
 
     MEMBER(audit_hooks),
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION == 13
     MEMBER(optimizer),
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
     MEMBER(executor_list_head), // TODO: Iterate this list
 #endif
 
@@ -218,8 +221,13 @@ static struct PyMemberDef ts_members[] = {
 
     MEMBER(context),
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION == 13
     MEMBER(previous_executor),
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 14
+    MEMBER(current_executor),
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
     MEMBER(threading_local_key),
     // threading_local_sentinel not included
 #endif
@@ -321,6 +329,10 @@ rootstate_relate(NyHeapRelate *r)
 #endif
 
         ISATTR(dict);
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        ISATTR(sysdict_copy);
+#endif
         ISATTR(builtins_copy);
 
 #ifdef HAVE_FORK
@@ -328,14 +340,13 @@ rootstate_relate(NyHeapRelate *r)
         ISATTR(after_forkers_parent);
         ISATTR(after_forkers_child);
 #endif
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 10
-        ISATTR(pyexitmodule);
-#endif
 
         ISATTR(audit_hooks);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION == 13
         ISATTR(optimizer);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
         ISATTR(executor_list_head);
 #endif
 
@@ -400,8 +411,13 @@ rootstate_relate(NyHeapRelate *r)
 
             TSATTR(context);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION == 13
             TSATTR(previous_executor);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 14
+            TSATTR(current_executor);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
             TSATTR(threading_local_key);
 #endif
         }
@@ -452,6 +468,10 @@ rootstate_traverse(NyHeapTraverse *ta)
 #endif
 
         Py_VISIT(is->dict);
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        Py_VISIT(is->sysdict_copy);
+#endif
         Py_VISIT(is->builtins_copy);
 
 #ifdef HAVE_FORK
@@ -459,14 +479,13 @@ rootstate_traverse(NyHeapTraverse *ta)
         Py_VISIT(is->after_forkers_parent);
         Py_VISIT(is->after_forkers_child);
 #endif
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 10
-        Py_VISIT(is->pyexitmodule);
-#endif
 
         Py_VISIT(is->audit_hooks);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION == 13
         Py_VISIT(is->optimizer);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
         Py_VISIT(is->executor_list_head);
 #endif
 
@@ -514,8 +533,13 @@ rootstate_traverse(NyHeapTraverse *ta)
 
             Py_VISIT(ts->context);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION == 13
             Py_VISIT(ts->previous_executor);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 14
+            Py_VISIT(ts->current_executor);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
             Py_VISIT(ts->threading_local_key);
 #endif
         }
@@ -735,6 +759,10 @@ rootstate_dir(PyObject *self, PyObject *args)
         ISATTR_DIR(codec_error_registry);
 
         ISATTR_DIR(dict);
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        ISATTR_DIR(sysdict_copy);
+#endif
         ISATTR_DIR(builtins_copy);
 
 #ifdef HAVE_FORK
@@ -742,14 +770,13 @@ rootstate_dir(PyObject *self, PyObject *args)
         ISATTR_DIR(after_forkers_parent);
         ISATTR_DIR(after_forkers_child);
 #endif
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 10
-        ISATTR_DIR(pyexitmodule);
-#endif
 
         ISATTR_DIR(audit_hooks);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION == 13
         ISATTR_DIR(optimizer);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
         ISATTR_DIR(executor_list_head);
 #endif
 
@@ -812,8 +839,13 @@ rootstate_dir(PyObject *self, PyObject *args)
 
             TSATTR_DIR(context);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION == 13
             TSATTR_DIR(previous_executor);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 14
+            TSATTR_DIR(current_executor);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
             TSATTR_DIR(threading_local_key);
 #endif
         }
