@@ -6,6 +6,7 @@
 #include "structmember.h"
 
 #include "../include/guppy.h"
+#include "../include/pythoncapi_compat.h"
 #include "../heapy/heapdef.h"
 #include "sets_internal.h"
 
@@ -264,7 +265,7 @@ NyMutNodeSet_SubtypeNewFlags(PyTypeObject *type, int flags, PyObject *hiding_tag
         return NULL;
     /*assert (flags & NS_HOLDOBJECTS); */
     v->flags = flags;
-    Py_SIZE(v) = 0;
+    Py_SET_SIZE(v, 0);
     v->u.bitset = (PyObject *)NyMutBitSet_New();
     if (!v->u.bitset) {
         Py_DECREF(v);
@@ -389,10 +390,10 @@ static void
 mutnodeset_dealloc(NyNodeSetObject *v)
 {
     PyObject_GC_UnTrack(v);
-    Py_TRASHCAN_SAFE_BEGIN(v)
+    Py_TRASHCAN_BEGIN(v, mutnodeset_dealloc)
     mutnodeset_gc_clear(v);
     Py_TYPE(v)->tp_free((PyObject *)v);
-    Py_TRASHCAN_SAFE_END(v)
+    Py_TRASHCAN_END
 }
 
 size_t
@@ -419,7 +420,7 @@ nodeset_traverse(NyHeapTraverse *ta)
 
 typedef struct {
     NyHeapRelate *r;
-    int i;
+    Py_ssize_t i;
 } RelateTravArg;
 
 static int
@@ -428,7 +429,7 @@ nodeset_relate_visit(PyObject *obj, RelateTravArg *ta)
     NyHeapRelate *r = ta->r;
 
     if (r->tgt == obj) {
-        r->visit(NYHR_RELSRC, PyUnicode_FromFormat("list(%%s)[%d]", ta->i), r);
+        r->visit(NYHR_INSET, PyLong_FromSsize_t(ta->i), r);
         return 1;
     }
     ta->i++;
@@ -605,7 +606,7 @@ NyNodeSet_setobj(NyNodeSetObject *v, PyObject *obj)
         if (r == -1)
             return -1;
         if (!r) {
-            Py_SIZE(v)++;
+            Py_SET_SIZE(v, Py_SIZE(v) + 1);
             if (v->flags & NS_HOLDOBJECTS) {
                 Py_INCREF(obj);
             }
@@ -627,7 +628,7 @@ NyNodeSet_clear(NyNodeSetObject *v)
         }
         if (NyMutBitSet_clear((NyMutBitSetObject *)v->u.bitset) == -1)
             return -1;
-        Py_SIZE(v) = 0;
+        Py_SET_SIZE(v, 0);
     } else {
         PyErr_Format(PyExc_ValueError,
                      "mutable nodeset required");
@@ -646,7 +647,7 @@ NyNodeSet_clrobj(NyNodeSetObject *v, PyObject *obj)
         if (r == -1)
             return -1;
         if (r) {
-            Py_SIZE(v)--;
+            Py_SET_SIZE(v, Py_SIZE(v) - 1);
             if (v->flags & NS_HOLDOBJECTS) {
                 Py_DECREF(obj);
             }
@@ -751,6 +752,7 @@ nodeset_pop(NyNodeSetObject *v, PyObject *argnotused)
         NyBit bitno = NyMutBitSet_pop((NyMutBitSetObject *)v->u.bitset, 0);
         if (bitno == -1 && PyErr_Occurred())
             return 0;
+        Py_SET_SIZE(v, Py_SIZE(v) - 1);
         return nodeset_bitno_to_obj(bitno);
     }
 }
